#include <iostream>
#include <string>
#include <map>
#include <strstream>

#include "preXmlNumber.hh"
#include "DWUnits.hh"
#include "ExpatInterface/Element.h"

namespace preXml {

  void preXml::Number::readFrom(const xml::Element & elt) {
    // Check we are reading the right kind of entity.
    assert(elt.getTagName() == string("number"));
    
    // These will be our allowed units:
    typedef std::map<std::string, double> UnitMap;
    UnitMap uM;
    uM[string("percent")]       = DWpercent;
    uM[string("dimensionless")] = DWdimensionless;
    uM[string("fraction" )]     = DWfraction;
    
    // Check we know this particular unit.
    string unit(elt[string("unit")]);
    assert(uM.find(unit) != uM.end());
    
    // OK, we know the unit.  We can go on.
    istrstream is(elt[string("value")].c_str());
    double faceValue;
    is >> faceValue;
    
    this->_value = faceValue*uM[unit];
  };
  
  void Number::printMeNicelyTo(ostream & os) const {
    os << this->_value / DWdimensionless;
  };


} // end of namespace
